@@ -13,14 +13,10 @@
 #' [shiny::reactiveValues()] object which you should keep for further use; in
 #' the examples and documentation, this object is typically called `ov`.
 #'
-#' This function also defines a dynamic output UI slot with ID
-#' `paste0(outputId, "_menu")`, which can be rendered using [shiny::renderUI()].
-#' When a user clicks the overlay's dropdown icon, this menu becomes visible
-#' and can be populated with inputs for editing overlay-specific settings, e.g.
-#' labels or numeric parameters tied to that overlay.
+#' @section `snap` parameter:
 #'
-#' If you provide a coordinate snapping function (`snap` argument), it should
-#' have the signature `function(ov, i)` where `ov` is the
+#' If you provide your own coordinate snapping function (`snap` argument), it
+#' should have the signature `function(ov, i)` where `ov` is the
 #' [shiny::reactiveValues()] object defining the overlays and their settings,
 #' and `i` is the set of indices for the rectangles to be updated. When the
 #' position of any of the overlays is changed, the snapping function will be
@@ -34,15 +30,72 @@
 #' one if the user might place an overlay onto the plot, but then change the x
 #' axis range of the plot such that the overlay is no longer visible. You can
 #' detect this by verifying whether the overlay rectangles are "out of bounds"
-#' at the top of your snapping function. See example below.
+#' at the top of your snapping function. See the code for [snapGrid()] for
+#' ideas.
+#'
+#' @section Overlay dropdown menu:
+#'
+#' Overlays have a little icon in the top-right corner (by default, a gear).
+#' When the user clicks on this icon, a dropdown menu appears that allows the
+#' user to remove the overlay. You can also provide additional components for
+#' this dropdown menu by using the `heading`, `select`, and `menu` parameters
+#' to [overlayServer()].
+#'
+#' **`heading`**: This should be a function with the signature `function(ov, i)`
+#' where `ov` is the [shiny::reactiveValues()] object defining the overlays and
+#' their settings, and `i` is the (single) index for the current overlay. The
+#' function should return a character string that will be used as the heading
+#' on thedropdown menu. This can be used to e.g. report the precise start and
+#' end point of the overlay, which may be useful to your users. The built-in
+#' functions [rangeHeading()] and [dateHeading()] can be used for numeric
+#' values and date values on the x-axis, respectively. Or you can use `NULL`
+#' for no heading on the dropdown menu.
+#'
+#' **`select`**: This can be `TRUE` to provide a [shiny::selectInput()] widget
+#' on the dropdown menu that users can use to change the type (i.e. label) of
+#' the current overlay. Or you can provide a character vector to restrict the
+#' widget to specific labels, or use `NULL` to omit this widget.
+#'
+#' **`menu`**: This can be a function with the signature `function(ov, i)`
+#' where `ov` is the [shiny::reactiveValues()] object defining the overlays and
+#' their settings, and `i` is the (single) index for the current overlay. It
+#' should return UI component(s) (if multiple components, wrapped in a `list`
+#' or `tagList`) that will be inserted into the dropdown menu. If you give the
+#' input widgets special IDs, the user can use those input widgets to directly
+#' modify certain properties of the overlays:
+#'
+#' \tabular{ll}{
+#' **`inputId`** \tab **Modifies** \cr
+#' \code{*_label} \tab The label of the overlay currently being edited. \cr
+#' \code{*_cx0} \tab Starting x-coordinate of overlay. \cr
+#' \code{*_cx1} \tab Ending x-coordinate of overlay. \cr
+#' \code{*_cx} \tab X-position of overlay; this is like `cx0`, but also updates `cx1` to keep the same width. \cr
+#' \code{*_cw} \tab Width of overlay; this adjusts `cx1` so that the overlay has the given width. \cr
+#' \code{*_XYZ} \tab The corresponding entry "XYZ" in `data` for the overlay being edited. \cr
+#' \tab Note: above, `*` stands for the `outputId` argument to `overlayServer()`.
+#' }
+#'
+#' See examples for an illustration of this.
 #'
 #' @param outputId The ID of the plot output (as used in [overlayPlotOutput()]).
 #' @param nrect Number of overlay rectangles to support.
 #' @param width Optional default overlay width in plot coordinates. If `NULL`
 #'     (default), set to 10% of the plot width.
-#' @param snap Function to "snap" overlay coordinates to a grid, or `"none"`
+#' @param data Named list of custom overlay-specific properties to be edited in
+#'     the overlay dropdown menu.
+#' @param snap Function to "snap" overlay coordinates to a grid, or `NULL`
 #'     (default) for no snapping. See details for how to specify the snap
-#'     function.
+#'     function; you can also use the built-in [snapGrid()].
+#' @param heading Function to provide a heading for the overlay dropdown menus,
+#'     or `NULL` (default) for no heading. See details for how to specify the
+#'     heading function; you can also use the built-in [rangeHeading()] or
+#'     [dateHeading()].
+#' @param select If you want to allow users to change the type (i.e. label) of
+#'     the overlay from the overlay dropdown menu, set this to `TRUE` to
+#'     provide a select input with all labels or a character vector with
+#'     permissible choices. `NULL` (default) to omit this feature.
+#' @param menu Function to provide additional UI elements on the overlay
+#'     dropdown menu. See details for how to specify the menu function.
 #' @param colours A function to assign custom colours to the overlays. Should
 #'     be a function that takes a single integer (the number of overlays) and
 #'     returns colours in hexadecimal notation (e.g. "#FF0000"). Do not provide
@@ -59,26 +112,29 @@
 #'     debugging purposes.
 #'
 #' @return A [shiny::reactiveValues()] object with the following named fields:
-#' \describe{
-#'   \item{n}{Number of overlays (read-only).}
-#'   \item{active}{Logical vector of length `n`; indicates which overlays are active.}
-#'   \item{show}{Logical vector; controls whether overlays are visible.}
-#'   \item{editing}{Index of the overlay currently being edited via the
-#'       dropdown menu, if any; `NA` otherwise (read-only).}
-#'   \item{last}{Index of the most recently added overlay (read-only).}
-#'   \item{snap}{Coordinate snapping function.}
-#'   \item{px, pw}{Numeric vector; overlay x-position and width in pixels (see note).}
-#'   \item{py, ph}{Numeric vector; overlay y-position and height in pixels (read-only).}
-#'   \item{cx0, cx1}{Numeric vector; overlay x-bounds in plot coordinates (see note).}
-#'   \item{label}{Character vector of labels shown at the top of each overlay.}
-#'   \item{outputId}{The output ID of the plot display area (read-only).}
-#'   \item{bound_cx, bound_cw}{x-position and width of the bounding area in plot coordinates (read-only).}
-#'   \item{bound_px, bound_pw}{x-position and width of the bounding area in pixels (read-only).}
-#'   \item{bound_py, bound_ph}{y-position and height of the bounding area in pixels (read-only).}
-#'   \item{stagger}{Amount of vertical staggering, as proportion of height.}
-#'   \item{style}{Named list of character vectors; additional styling for rectangular overlays.}
-#'   \item{update_cx(i)}{Function to update `cx0`/`cx1` from `px`/`pw` for overlays `i` (see note).}
-#'   \item{update_px(i)}{Function to update `px`/`pw` from `cx0`/`cx1` for overlays `i` (see note).}
+#' \tabular{ll}{
+#' \code{n} \tab Number of overlays (read-only). \cr
+#' \code{show} \tab `TRUE`/`FALSE`; controls whether overlays are visible. \cr
+#' \code{active} \tab Logical vector of length `n`; indicates which overlays are active. \cr
+#' \code{label} \tab Character vector of labels shown at the top of each overlay. \cr
+#' \code{data} \tab Custom data for each overlay, to be edited via the dropdown menu. \cr
+#' \code{editing} \tab Index of the overlay currently being edited via the dropdown menu; `NA` if none (read-only). \cr
+#' \code{last} \tab Index of the most recently added overlay (read-only). \cr
+#' \code{snap} \tab Coordinate snapping function. \cr
+#' \code{heading} \tab Heading function for the dropdown menu. \cr
+#' \code{select} \tab Overlay label select options for the dropdown menu. \cr
+#' \code{menu} \tab Function to provide additional UI elements for the dropdown menu. \cr
+#' \code{px,pw} \tab Numeric vector; overlay x-position and width in pixels (see note). \cr
+#' \code{py,ph} \tab Numeric vector; overlay y-position and height in pixels (read-only). \cr
+#' \code{cx0,cx1} \tab Numeric vector; overlay x-bounds in plot coordinates (see note). \cr
+#' \code{outputId} \tab The output ID of the plot display area (read-only). \cr
+#' \code{bound_cx, bound_cw} \tab x-position and width of the bounding area in plot coordinates (read-only). \cr
+#' \code{bound_px, bound_pw} \tab x-position and width of the bounding area in pixels (read-only). \cr
+#' \code{bound_py, bound_ph} \tab y-position and height of the bounding area in pixels (read-only). \cr
+#' \code{stagger} \tab Amount of vertical staggering, as proportion of height. \cr
+#' \code{style} \tab Named list of character vectors; additional styling for rectangular overlays. \cr
+#' \code{update_cx(i)} \tab Function to update `cx0`/`cx1` from `px`/`pw` for overlays `i` (see note). \cr
+#' \code{update_px(i)} \tab Function to update `px`/`pw` from `cx0`/`cx1` for overlays `i` (see note).
 #' }
 #'
 #' Note: Fields marked "read-only" above should not be changed. Other fields can
@@ -95,44 +151,31 @@
 #' the overlay(s) to be updated.
 #'
 #' @examples
-#' # Example of a valid snapping function: snap to nearest round number and
-#' # make sure the overlay is at least 2 units wide.
-#' mysnap <- function(ov, i) {
-#'     # remove any "out of bounds" overlays
-#'     oob <- seq_len(ov$n) %in% i &
-#'         (ov$cx0 < ov$bound_cx | ov$cx1 > ov$bound_cx + ov$bound_cw)
-#'     ov$active[oob] <- FALSE
-#'
-#'     # adjust position and with
-#'     widths <- pmax(2, round(ov$cx1[i] - ov$cx0[i]))
-#'     ov$cx0[i] <- pmax(round(ov$bound_cx),
-#'         pmin(round(ov$bound_cx + ov$bound_cw) - widths, round(ov$cx0[i])))
-#'     ov$cx1[i] <- pmin(round(ov$bound_cx + ov$bound_cw), ov$cx0[i] + widths)
-#' }
-#'
 #' ui <- shiny::fluidPage(
-#'     useOverlay(),
 #'     overlayPlotOutput("my_plot", 640, 480),
 #'     overlayToken("add", "Raise")
 #'     # further UI elements here . . .
 #' )
 #'
 #' server <- function(input, output) {
-#'     ov <- overlayServer("my_plot", 4, 1, snap = mysnap)
+#'     menu <- function(ov, i) {
+#'         sliderInput("my_plot_amount", "Raise amount",
+#'                value = ov$data$amount[i], min = 0, max = 1)
+#'     }
 #'
-#'     output$my_plot_menu <- renderUI({
-#'         i <- req(ov$editing)
-#'         textInput("label_input", "Overlay label", value = ov$label[i])
-#'     })
-#'
-#'     observeEvent(input$label_input, {
-#'         i <- req(ov$editing)
-#'         ov$label[i] <- input$label_input
-#'     })
+#'     ov <- overlayServer("my_plot", 4, 1,
+#'         data = list(amount = 0.2),
+#'         snap = snapGrid(step = 0.1),
+#'         heading = rangeHeading(digits = 3),
+#'         menu = menu)
 #'
 #'     output$my_plot <- shiny::renderPlot({
 #'         df <- data.frame(x = seq(0, 2 * pi, length.out = 200))
-#'         df$y <- sin(df$x) + 0.1 * sum(ov$active * (df$x > ov$cx0 & df$x < ov$cx1))
+#'         df$y <- (1 + sin(df$x)) / 2
+#'         for (i in which(ov$active)) {
+#'             xi <- (df$x >= ov$cx0[i] & df$x <= ov$cx1[i])
+#'             df$y[xi] <- df$y[xi] + ov$data$amount[i]
+#'         }
 #'         plot(df, type = "l")
 #'         overlayBounds(ov, "base")
 #'     })
@@ -146,10 +189,13 @@
 #' @seealso [overlayPlotOutput()], [overlayBounds()]
 #'
 #' @export
-overlayServer = function(outputId, nrect, width = NULL, snap = "none",
+overlayServer = function(outputId, nrect, width = NULL, data = NULL,
+    snap = NULL, heading = NULL, select = NULL, menu = NULL,
     colours = overlayColours, opacity = 0.25, icon = shiny::icon("gear"),
     stagger = 0.045, style = list(), debug = FALSE)
 {
+    # ---------- VALIDATE PARAMETERS ----------
+
     session = shiny::getDefaultReactiveDomain()
     input = session$input
     output = session$output
@@ -173,16 +219,41 @@ overlayServer = function(outputId, nrect, width = NULL, snap = "none",
         })
     }
 
-    # ---------- GLOBAL SETUP ----------
+    # Validate data parameter
+    if (length(data) > 0 &&
+            (!is.list(data) || is.null(names(data)) ||
+            any(is.na(names(data))) || any(names(data) == ""))) {
+        stop("Data must be a named list or NULL.")
+    }
+    data = lapply(data, function(x) {
+        if (length(x) == 1) {
+            rep(x, nrect)
+        } else if (length(x) == nrect) {
+            x
+        } else {
+            stop("All elements of data must be either length 1 or length nrect (", nrect, ").")
+        }
+    })
+
+    # ---------- ENABLE TOKENS ----------
 
     # Intervention tokens
     shinyjqui::jqui_draggable(ui = ovmatch("token"),
         options = list(revert = TRUE, helper = "clone", opacity = 0.75, revertDuration = 0, zIndex = 9999));
 
+    # Refresh token list
+    refresh_tokens = function() {
+        shinyjs::runjs(
+'var token_labels = $(".overshiny-token").map(function() { return $(this).attr("data-label"); }).get();
+Shiny.setInputValue("overshiny_tokens", token_labels);'
+        )
+    }
+    refresh_tokens()
+
     # TODO Monitor resizing of plot
     # shinyjs::runjs(paste0('observePlotResize("', outputId, '")'));
 
-    # ---------- OVERLAY SETUP ----------
+    # ---------- ENABLE OVERLAYS ----------
 
     # Run setup code for the display
     bounds = ovid("bounds", outputId)
@@ -206,21 +277,27 @@ overlayServer = function(outputId, nrect, width = NULL, snap = "none",
         )
     }
 
+    # ---------- OVERLAY OBJECT ----------
+
     # Set up overlays
     ov = shiny::reactiveValues(
         n         = nrect,          # number of overlays
-        active    = rep(F, nrect),  # is the overlay active
         show      = TRUE,           # show overlays?
+        active    = rep(F, nrect),  # is the overlay active
+        label     = rep("", nrect), # label at top of overlay
+        data      = data,           # user settings
         editing   = NA,             # which overlay is currently being edited via dropdown
         last      = NA,             # which overlay was last to be added
         snap      = snap,           # coordinate snapping function
+        heading   = heading,        # overlay menu heading function
+        select    = select,         # overlay menu select for label
+        menu      = menu,           # overlay menu UI function
         px        = rep(0, nrect),  # left pixel position of overlay
         pw        = rep(0, nrect),  # pixel width of overlay
         py        = rep(0, nrect),  # bottom pixel position of overlay
         ph        = rep(0, nrect),  # pixel height of overlay
         cx0       = rep(0, nrect),  # left x coord of overlay
         cx1       = rep(1, nrect),  # right x coord of overlay
-        label     = rep("", nrect), # label at top of overlay
         outputId  = outputId,       # id of display/bounds
         bound_cx  = 0,              # x-pos of bounds in coords (set by overlayBounds)
         bound_cw  = 0,              # width of bounds in coords (set by overlayBounds)
@@ -238,14 +315,14 @@ overlayServer = function(outputId, nrect, width = NULL, snap = "none",
         ov$cx0[i] = (ov$px[i] / ov$bound_pw) * ov$bound_cw + ov$bound_cx;
         ov$cx1[i] = ((ov$px[i] + ov$pw[i]) / ov$bound_pw) * ov$bound_cw + ov$bound_cx;
 
-        if (!identical(ov$snap, "none")) {
+        if (!is.null(ov$snap)) {
             ov$update_px(i)
         }
     }
 
     # Set px and pw of all overlays from cx0 and cx1
     ov$update_px = function(j = seq_len(ov$n)) {
-        if (identical(ov$snap, "none")) {
+        if (is.null(ov$snap)) {
             # Ensure times are in proper range
             ov$cx0[j] = pmax(ov$cx0[j], ov$bound_cx);
             ov$cx1[j] = pmin(ov$cx1[j], ov$bound_cx + ov$bound_cw);
@@ -263,6 +340,8 @@ overlayServer = function(outputId, nrect, width = NULL, snap = "none",
         }
     }
 
+    # ---------- OVERLAY REACTIVITY ----------
+
     # Make overlays respond to ov$active and ov$show
     shiny::observe({
         for (i in seq_len(ov$n)) {
@@ -271,6 +350,13 @@ overlayServer = function(outputId, nrect, width = NULL, snap = "none",
         }
     })
 
+    # Make overlays respond to ov$label
+    shiny::observe({
+        for (i in seq_len(ov$n)) {
+            shinyjs::html(ovid("label", outputId, i), ov$label[i]);
+        }
+    })
+
     # Make overlays respond to ov$style
     shiny::observe({
         for (i in seq_len(ov$n)) {
@@ -285,12 +371,102 @@ overlayServer = function(outputId, nrect, width = NULL, snap = "none",
         ov$update_px()
     })
 
-    # Close all dropdowns and their contents
+    # ---------- OVERLAY DROPDOWN ----------
+
+    # Helper to insert UI for overlay dropdown menu
+    insert_dropdown = function(i) {
+        # Dropdown menu components
+        heading = if (!is.null(ov$heading)) {
+            shiny::textOutput(ovid("heading"))
+            output[[ovid("heading")]] = shiny::renderText({ ov$heading(ov, i) });
+        }
+        select = if (!is.null(ov$select)) {
+            if (isTRUE(ov$select)) {
+                choices = input$overshiny_tokens
+            } else if (is.character(ov$select)) {
+                choices = ov$select
+            } else {
+                stop("select must be TRUE or a character vector of overlayToken labels.")
+            }
+            shiny::selectInput(ovid("select"),
+                label = NULL,
+                choices = choices,
+                selected = ov$label[i])
+        }
+        extra = if (!is.null(ov$menu)) {
+            shiny::isolate(ov$menu(ov, i))
+        }
+        trash = shiny::actionLink(inputId = ovid("remove"), label = "Remove",
+            icon = shiny::icon("trash"), class = "overshiny-remove",
+            `data-id` = ovid("overlay", outputId, i))
+
+        # Insert dropdown menu
+        insert_ui(ovid("dropdown", outputId, i),
+            htmltools::div(id = ovid("menu"),
+                heading, select, extra, trash))
+    }
+
+    # Helper to close all dropdowns and remove their contents
     clear_dropdowns = function() {
         setcss(ovmatch("dropdown", outputId), display = "none")
         shiny::removeUI(ovsel("menu"), immediate = TRUE)
     }
 
+    # Observe built-in dropdown label select
+    shiny::observeEvent(input[[ovid("select")]], {
+        i = shiny::req(ov$editing)
+        ov$label[i] = input[[ovid("select")]]
+        shiny::removeUI(ovsel("menu"), immediate = TRUE)
+        insert_dropdown(i)
+    })
+
+    # Observe "extra" menu-related input changes
+    data_head = paste0("^", outputId, "_")
+    shiny::observe({
+        names = names(input)
+        names = names[grepl(data_head, names)]
+        for (nm in names) {
+            input[[nm]] # take dependency
+        }
+
+        i = shiny::req(shiny::isolate(ov$editing))
+
+        names = names(input)
+        names = names[grepl(data_head, names)]
+        for (nm in names) {
+            inm = stringr::str_remove(nm, data_head)
+            if (inm == "label") {
+                ov$label[[i]] = input[[nm]]
+            } else if (inm == "cx0") {
+                shiny::isolate({
+                    ov$cx0[[i]] = input[[nm]]
+                    ov$update_px(i)
+                })
+            } else if (inm == "cx1") {
+                shiny::isolate({
+                    ov$cx1[[i]] = input[[nm]]
+                    ov$update_px(i)
+                })
+            } else if (inm == "cx") {
+                shiny::isolate({
+                    w = ov$cx1[[i]] - ov$cx0[[i]]
+                    ov$cx0[[i]] = input[[nm]]
+                    ov$cx1[[i]] = input[[nm]] + w
+                    ov$update_px(i)
+                })
+            } else if (inm == "cw") {
+                shiny::isolate({
+                    ov$cx1[[i]] = ov$cx0[[i]] + input[[nm]]
+                    ov$update_px(i)
+                })
+            } else if (!is.null(ov$data[[inm]])) {
+                ov$data[[inm]][[i]] = input[[nm]]
+            }
+        }
+    })
+
+    # ---------- OVERLAY EVENTS ----------
+
     # Observe move, resize, dropdown, remove, defocus, plot_size, debug events
     shiny::observeEvent(input$overshiny_event, {
         # Extract rectangle id
@@ -301,43 +477,28 @@ overlayServer = function(outputId, nrect, width = NULL, snap = "none",
 
         if (input$overshiny_event$what == "move") {
             # Move: update overlay's pixel position, then coordinate info
-            shiny::isolate({
-                ov$px[i] = input$overshiny_event$x
-                ov$update_cx(i)
-            })
+            ov$px[i] = input$overshiny_event$x
+            ov$update_cx(i)
         } else if (input$overshiny_event$what == "resize") {
             # Resize: update overlay's pixel position & width, then coordinate info
-            shiny::isolate({
-                ov$px[i] = input$overshiny_event$x
-                ov$pw[i] = input$overshiny_event$w
-                ov$update_cx(i)
-            })
+            ov$px[i] = input$overshiny_event$x
+            ov$pw[i] = input$overshiny_event$w
+            ov$update_cx(i)
         } else if (input$overshiny_event$what == "dropdown") {
             # Dropdown menu
             clear_dropdowns()
             # Insert and make visible new dropdown
-            if (!isTRUE(shiny::isolate(ov$editing) == i)) {
-                shiny::isolate({
-                    ov$editing = i;
-                    insert_ui(ovid("dropdown", outputId, i),
-                        ui = htmltools::div(id = ovid("menu"),
-                            shiny::uiOutput(paste(outputId, "menu", sep = "_")),
-                            shiny::actionButton(inputId = "int_remove", label = "Remove",
-                                icon = shiny::icon("trash"), class = "overshiny-remove",
-                                `data-id` = ovid("overlay", outputId, i)
-                            )
-                        )
-                    );
-                })
+            if (!isTRUE(ov$editing == i)) {
+                ov$editing = i
+                insert_dropdown(i)
                 setcss(ovid("dropdown", outputId, i), display = "block")
             } else {
-                ov$editing = NA;
+                # Close menu
+                ov$editing = NA
             }
         } else if (input$overshiny_event$what == "remove") {
-            shiny::isolate({
-                ov$active[i] = F;
-                ov$editing = NA;
-            })
+            ov$active[i] = F;
+            ov$editing = NA;
             clear_dropdowns()
         } else if (input$overshiny_event$what == "defocus") {
             ov$editing = NA;
@@ -358,132 +519,25 @@ overlayServer = function(outputId, nrect, width = NULL, snap = "none",
 
             # Set overlay position and label and mark as active
             default_width = if (is.null(width)) 0.1 * ov$bound_pw else width
-            shiny::isolate({
-                ov$editing = NA;
-                ov$last = i;
-                ov$pw[i] = min(ov$bound_pw, default_width * ov$bound_pw / ov$bound_cw);
-                ov$px[i] = max(0, min(ov$bound_pw - ov$pw[i], input[[drop_event]]$x - ov$bound_px));
-                ov$update_cx(i);
-                ov$active[i] = TRUE;
-                ov$label[i] = input[[drop_event]]$label;
+            ov$editing = NA;
+            ov$last = i;
+            ov$pw[i] = min(ov$bound_pw, default_width * ov$bound_pw / ov$bound_cw);
+            ov$px[i] = max(0, min(ov$bound_pw - ov$pw[i], input[[drop_event]]$x - ov$bound_px));
+            ov$update_cx(i);
+            ov$active[i] = TRUE;
+            ov$label[i] = input[[drop_event]]$label;
 
-                # Position overlay
-                setcss(ovid("overlay", outputId, i),
-                    left = ov$px[i], width = ov$pw[i],
-                    bottom = ov$py[i], height = ov$ph[i]);
-            })
+            # Reset data
+            for (nm in names(ov$data)) {
+                ov$data[[nm]][[i]] = data[[nm]][[i]]
+            }
+
+            # Position overlay
+            setcss(ovid("overlay", outputId, i),
+                left = ov$px[i], width = ov$pw[i],
+                bottom = ov$py[i], height = ov$ph[i]);
         }
     });
 
-    # Change labels on overlays as needed
-    shiny::observeEvent(ov$label, {
-        shiny::isolate({
-            for (i in seq_len(ov$n)) {
-                shinyjs::html(ovid("label", outputId, i), ov$label[i]);
-            }
-    })})
-
     return (ov)
 }
-
-#' Align overlays with a ggplot2 or base plot
-#'
-#' Sets the pixel and coordinate bounds of the overlay area based on a
-#' [ggplot2::ggplot()] object or base R plot. This ensures that overlays are
-#' positioned correctly in both visual and coordinate space.
-#'
-#' Call this function within [shiny::renderPlot()], before returning the
-#' ggplot object (if using ggplot2) or `NULL` (if using base R plotting).
-#'
-#' @param ov A [shiny::reactiveValues()] object returned by [overlayServer()].
-#' @param plot A [ggplot2::ggplot()] object used for overlay alignment, or the
-#'     character string `"base"` if you are using base R plotting.
-#' @param xlim,ylim Vectors defining the coordinate limits for overlays.
-#'     Use `NA` to inherit axis limits from the plot panel.
-#' @param row,col Row and column of the facet panel (if applicable). This only
-#'     works with ggplot2 plots; base R plots with multiple panels are not
-#'     supported.
-#'
-#' @return The ggplot object (for ggplot2) or `NULL` (for base R plotting), to
-#' be returned from the [shiny::renderPlot()] block.
-#'
-#' @examples
-#' server <- function(input, output) {
-#'     ov <- overlayServer("my_plot", 1, 1)
-#'     output$my_plot <- shiny::renderPlot({
-#'         plot(1:100, sin(1:100 * 0.1), type = "l")
-#'         overlayBounds(ov, "base", xlim = c(1, 100))
-#'     })
-#'     # further server code here . . .
-#' }
-#'
-#' @seealso [overlayServer()], for a complete example.
-#'
-#' @export
-overlayBounds = function(ov, plot, xlim = c(NA, NA), ylim = c(NA, NA), row = 1L, col = 1L)
-{
-    input = shiny::getDefaultReactiveDomain()$input
-
-    if (ggplot2::is_ggplot(plot)) {
-        rect = panel_rects_ggplot(plot)
-    } else if (identical(plot, "base")) {
-        rect = panel_rects_base()
-    } else {
-        stop("Unrecognised plot type")
-    }
-
-    rect = rect[rect$row == row & rect$col == col]
-    if (nrow(rect) != 1) {
-        stop("Invalid row, col")
-    }
-
-    # Adjust NA to bounds of plot
-    if (is.na(xlim[1])) xlim[1] = rect$xmin
-    if (is.na(xlim[2])) xlim[2] = rect$xmax
-    if (is.na(ylim[1])) ylim[1] = rect$ymin
-    if (is.na(ylim[2])) ylim[2] = rect$ymax
-    xlim = as.numeric(xlim)
-    ylim = as.numeric(ylim)
-
-    # bx, bw: x and w of bounds in normalised image coordinates (0 to 1)
-    bx = rect$x + rect$w * (xlim[1] - rect$xmin) / (rect$xmax - rect$xmin)
-    bw = rect$w * (xlim[2] - xlim[1]) / (rect$xmax - rect$xmin)
-
-    # by, bh: similar
-    by = rect$y + rect$h * (ylim[1] - rect$ymin) / (rect$ymax - rect$ymin)
-    bh = rect$h * (ylim[2] - ylim[1]) / (rect$ymax - rect$ymin)
-
-    # Get width and height of target plot, plus left offset of plot
-    outputId = shiny::isolate(ov$outputId)
-    shinyjs::runjs(paste0(
-        'var plot = $("#', outputId, '");\n',
-        'Shiny.setInputValue("overshiny_return",',
-        ' [plot.width(), plot.height(), plot.offset().left]);'
-    ))
-    shiny::req(input$overshiny_return)
-    img_width = input$overshiny_return[1];
-    img_height = input$overshiny_return[2];
-    left_offset = input$overshiny_return[3];
-
-    l = bx * img_width
-    w = bw * img_width
-    b = by * img_height
-    h = bh * img_height
-
-    shiny::isolate({
-        ov$bound_cx = xlim[1]
-        ov$bound_cw = xlim[2] - xlim[1]
-        ov$bound_px = l + left_offset
-        ov$bound_pw = w
-        ov$bound_py = b
-        ov$bound_ph = h
-        ov$py = rep(0, ov$n)
-        ov$ph = h * (1 - 1:ov$n * ov$stagger)
-        ov$update_px() # in case bounds have changed
-    })
-
-    setcss(ovid("bounds", outputId), left = paste0(l, "px"), bottom = paste0(b, "px"),
-        width = paste0(w, "px"), height = paste0(h, "px"))
-
-    return (plot)
-}
